function [Z,Sk] = HO_Z_CV(priors,draw)
Z = draw.Z;
%   This program draws the aggregate state vector Z
%   Here, Z can take on values from 1 to K
%   The draws are generated from a multi-move Gibbs sampler
%   draw.P are the transition probabilities that sum to one in a column
%   Compute the steady state probabilities based on the transition probabilities
Zhold = draw.Z;
prSS = [];%these are the steady state probabilities
Pr_Zt_Ztm1 = draw.P;
for i = 1:priors.K;
    pS = (priors.K - 1 - trace(draw.P) + draw.P(i,i))/(priors.K - trace(draw.P));
    prSS = [prSS; pS];
end,
Pr_Ztm1_tm1 = prSS;
%*************************************************************************   
%    Filter
%*************************************************************************
Pr_Zt_t = zeros(priors.K,priors.T);
for t = 1:priors.T
    %   Hamilton Filter
    %   First compute the probability of observing the combination Zt,Ztm1 conditional on observations from tm1  
    %   Pr_Zt_Ztm1_tm1 = Pr_Zt_Ztm1 * Pr_Ztm1_tm1
    Pr_Zt_Ztm1_tm1 = zeros(priors.K,priors.K);
    %disp(Pr_Ztm1_tm1)
    for k = 1:priors.K
        Pr_Zt_Ztm1_tm1(k,:) = Pr_Zt_Ztm1(k,:) .* Pr_Ztm1_tm1';
    end,
    %disp(Pr_Zt_Ztm1_tm1);
    %   Next compute the forecast error for each possibility of Z
    log_f_yt_Zt_Ztm1_tm1 = zeros(k,1);
    OMEGA = diag(draw.sigsq);
    %term1 = sqrt(2 * pi * det(OMEGA));
    for k = 1:priors.K
        %   in order to compute the forecast error, we need to change from Z to S
        fcast = priors.YY(t,:)' - draw.mu_0 - draw.mu_1 .* draw.H(:,k);
        term2 = -0.5 * fcast' * inv(OMEGA) * fcast;
        %   this is the nonconstant portion of the log likelihood
        log_f_yt_Zt_Ztm1_tm1(k,:) = term2;
        %f_yt_Zt_Ztm1_tm1(k,:) = (1 / term1) * term2;
    end,
    scale = min(log_f_yt_Zt_Ztm1_tm1) - 1;
    f_yt_Zt_Ztm1_tm1 = exp(log_f_yt_Zt_Ztm1_tm1 - scale * ones(priors.K,1));
    f_prime = repmat(f_yt_Zt_Ztm1_tm1,1,priors.K);
    %   Pr_Zt_Ztm1_t = f_yt_Zt_Ztm1_tm1 * Pr_Zt_Ztm1_tm1) / f_yt_tm1
    %   f_yt_tm1 = sum(sum(f_yt_Zt_Ztm1_tm1 * Pr_Zt_Ztm1_tm1))
    pr_zz_t = f_prime .* Pr_Zt_Ztm1_tm1;
    f_yt_tm1 = sum(sum(pr_zz_t,1),2);
    Pr_Zt_Ztm1_t = pr_zz_t / f_yt_tm1;
    %   Pr_Zt_t = sum ( Pr_Zt_Ztm1_t )
    Pr_Zt_t(:,t) = sum(Pr_Zt_Ztm1_t,2);
    Pr_Ztm1_tm1 = Pr_Zt_t(:,t);
end,
%   generate for time T
u = unifrnd(0,1) * ones(priors.K,1);
Pr_ = Pr_Zt_t(:,priors.T) / sum(Pr_Zt_t(:,priors.T));
PR_ = cumsum(Pr_);
Z(priors.T,:) = find(le(u,cumsum(PR_)),1);
acc = 0;wait = 0;tries = 0;
while acc == 0
    for t = priors.T-1:-1:1
        %   Generation step
        %   For t = T-1, ..., 1, we want to multiply the filtered probability by the forward transition
        %   probability:  Pr_Zt_Ztp1_t = Pr_Ztp1_Zt * Pr_Zt_t
        %   In this case, we know Ztp1 (generated from the last step) so Pr_Ztp1_Zt is the appropriate row
        %   from the transition matrix
        Pr_Ztp1_Zt = draw.P(Z(t+1,:),:);
        Pr_Zt_Ztp1_t = Pr_Ztp1_Zt' .* Pr_Zt_t(:,t);

        if sum(Pr_Zt_Ztp1_t) == 0
            t = priors.T-1;
            tries = tries + 1;
        else
            PR_ = Pr_Zt_Ztp1_t / sum(Pr_Zt_Ztp1_t);
            u = unifrnd(0,1) * ones(priors.K,1);
            Z(t,:) = find(le(u,cumsum(PR_)),1);
        end,
        if tries > 100
            error('stop')
        end
    end,
    z = zeros(priors.T,priors.K);
    for t = 1:priors.T
        z(t,Z(t,:)) = 1;
    end,
    if all(sum(z,1) > 2)
        acc = 1;
    else
        wait = wait + 1;
    end
    if wait > 100
        acc = 1;
        Z = Zhold;
    end,
end,

%   Based on draw.Z, we can form a series Sk, which is the binary variable for each cluster
%   The previous model allowed only one cluster.  In this case, a state can belong to more than one cluster
%   As the previous incarnation, Z = 1 is everyone's in expansion
%                                Z = 2 is everyone's in recession
Sk = zeros(priors.T,priors.K);
for i = 2:priors.K
    Sk(:,i) = (draw.Z == i);
end





