
function [priors,results,draw] = HO_c_priors_CV(KAPPA)
%   ***************************************************************************
%   Parameters for the sampler
priors.Nburn = 250000;priors.Nkeep = 25000;
priors.datanam = 'State Data with shares';
%   ***************************************************************************
%   Location of the data
priors.mainfile = 'Employment_gr';
priors.savefile = 'C:\Users\mowyang\Desktop\HO Cross Val New 1\emp growth res';
priors.location = 'C:\Users\rb\Desktop\Hamilton and Owyang RESTAT Code';
priors.loadfile = [priors.location,'\states_qtr_031908.xls'];
priors.covariatefile = 'Covariates';
%   ***************************************************************************
%   These load the data
priors.YY = xlsread(priors.loadfile,priors.mainfile);
[priors.T,priors.N] = size(priors.YY);
priors.K = KAPPA + 2;       %this is the number of aggregate states
%   ***************************************************************************
%   Load the covariates here
priors.x = xlsread(priors.loadfile,priors.covariatefile);
priors.x = [ones(size(priors.x,1),1),priors.x];
priors.pp = size(priors.x,2);

%   ***************************************************************************
%   ***************************************************************************
%   Outlier filter the data
for n = 1:priors.N
    pp = find(abs(priors.YY(:,n) - mean(priors.YY(:,n),1)) > 3 * std(priors.YY(:,n),1));
    for i = 1:size(pp)
        priors.YY(pp(i),n) = mean(priors.YY(:,n),1) + sign(priors.YY(pp(i),n) - mean(priors.YY(:,n),1)) * 2 * std(priors.YY(:,n),1);
    end,
end
%   ***************************************************************************
%   ***************************************************************************
%	Priors for the estimation
%   ***************************************************************************
%   mu_0, mu_1 have a normal prior (these are the same for each state)
priors.t0_mu = [1;-2];   %this is the mean of the mu prior (2 x 1)
priors.R0_mu = eye(2);   %this is the var of the mu prior (2 x 2)
priors.nu = 0;
priors.d = 0;
%   dirichlet priors for the transition matrix
priors.p = zeros(priors.K,priors.K); %needs to be (K x K)
%   this matrix disables transitions between some states
priors.P_restrict = ones(priors.K,priors.K);
priors.P_restrict(3:end,3:end) = eye(priors.K - 2);
%   ***************************************************************************
%   Priors for the multinomial
%   ***************************************************************************
priors.B = 0.5 * eye(priors.pp);   %   the covariance matrix for beta
priors.b = zeros(priors.pp,priors.K - 2);           %   the mean for beta
%   ***************************************************************************
%   ***************************************************************************
%	Initialize the draw structure
%   ***************************************************************************
%   the parameters
priors.no_clust = KAPPA;
draw.mu_0 =  5 * ones(priors.N,1);
draw.mu_1 = -10 * ones(priors.N,1);
draw.sigsq = 10 * ones(priors.N,1);
%   the transition matrix
draw.P = 0.9 * eye(priors.K) + [(0.1 / priors.K) * ones(priors.K,2) zeros(priors.K,priors.K - 2)];
draw.P = draw.P + [zeros(priors.K,2) [0.05 * ones(2,priors.K - 2);zeros(priors.K-2,priors.K-2)]];
%   the aggegate and state-level regimes
draw.Z = unidrnd(priors.K,priors.T,1);
priors.Z_init = draw.Z;

%   probably we should use some intuition about starting values here....
draw.KAY = zeros(priors.N,priors.no_clust);% binornd(1,0.5,priors.N,priors.no_clust);
%   ***************************************************************************
%   Initialize the beta, lam matrices
%       beta is a pp x K-2 matrix, where pp is the number of covariates and K-2 is the number of clusters
draw.beta = zeros(priors.pp,priors.K-2);
% %   Initialize the z, lam matrix
draw.z = zeros(priors.N,priors.K-2);
for k = 1:priors.K-2
    draw.lam(:,k) = ones(priors.N,1);
    for n = 1:priors.N
        accept = 0;
        while accept == 0
            U = rand;
            draw.z(n,k) = - log(inv(U) - 1);
            if draw.z(n,k) * (draw.KAY(n,k) - 0.5) > 0
                accept = 1;
            end,
        end,
    end,
end,
%   ***************************************************************************
%   the default i'm using here is that the degenerate cluster is K-1
draw.H = [zeros(priors.N,1) ones(priors.N,1) zeros(priors.N,priors.K-2)];
%   ***************************************************************************
draw.Sk = zeros(priors.T,priors.no_clust);
for i = 2:priors.K
    draw.Sk(:,i) = (draw.Z == i);
end,
draw.Sk(:,priors.no_clust) = (draw.Z == 2);
results = [];
%   ***************************************************************************
%   Form the results structure
%   ***************************************************************************
results.ZED = zeros(priors.T,priors.K,priors.Nkeep);
results.mu_0 = zeros(priors.N,priors.Nkeep);
results.mu_1 = zeros(priors.N,priors.Nkeep);
results.sigsq = zeros(priors.N,priors.Nkeep);
results.P = zeros(priors.K,priors.K,priors.Nkeep);
results.KAY = zeros(priors.N,priors.K-2,priors.Nkeep);
results.beta = zeros(priors.pp,priors.K-2,priors.Nkeep);






