%   HO_clusters_CV                  07/14/2008, last modified 10/12/2010
%   This is the HO_clusters model with kappa = number of clusters set endogenously

%   The algorith uses the following constructions:
%       mu_0, mu_1, sigsq are N x 1 vectors
%       H is a (priors.N x priors.K) matrix of zeros and ones which indicates recession (h(ik) = 1) and expansions
%       H is equal to [zeros(priors.N,1) ones(priors.N,1) Hprime]
%       Hprime is a matrix that indicates which cluster the state is in (this is not used in the program)
%       If the nth row is all zeros, this indicates that the state is in the degenerate cluster
%       Z is the (T x 1) vector of aggregate regimes
%       Any element of Z takes on a value of 1 to K
%       We'll store something called ZED, which is Z converted to zeros and ones

%       The nth state's period-t business cycle regime is the nth row, Z(t)-th column of H
%       For each cluster, we can create a state vector.  The compilation of these we'll call Sk
%       Then we'll define a variable called draw.k which is a summary of the state
%       draw.Sk and draw.k are redundant with draw.Z and draw.H
%       The convention we use here is that the degenerate cluster is the kappa-th cluster (i.e., last)
%
%       We want to restrict the movement between some aggregate regimes
%       P is a K x K matrix of transition probabilities
%       P_restict is a matrix that helps set some of the elements of P equal to zero (this is set by user)
%       The convention here is that the row indicates the period t and columns indicate period t-1



function [priors,draw,results,finalres,meanz] = HO_clusters_CV(KAPPA)

%	Input the data and the priors
[priors,results,draw] = HO_c_priors_CV(KAPPA);
%   Burn in iterations
for i = 1:priors.Nburn
    disp(['Burn Iteration ',num2str(i)])
    %    Draw the cluster
    [draw.H,draw.KAY,draw.beta,draw.lam,draw.z] = HO_H_CV(priors,draw);
    %    Draw mu0,mu1,sigsq for each state
    [draw.mu_0,draw.mu_1,draw.sigsq] = HO_THETA_CV(priors,draw);
    %    Draw the aggregate regime process Z conditional on H
    [draw.Z,draw.Sk] = HO_Z_CV(priors,draw);
    %    Draw the transition probabilities conditional on Z
    [draw.P] = HO_P_CV(priors,draw);

end,
%   Form the posterior distribution
for i = 1:priors.Nkeep
    disp(['Save Iteration ',num2str(i)])
    %    Draw the cluster
    [draw.H,draw.KAY,draw.beta,draw.lam,draw.z] = HO_H_CV(priors,draw);
    %    Draw mu0,mu1,sigsq for each state
    [draw.mu_0,draw.mu_1,draw.sigsq] = HO_THETA_CV(priors,draw);
    %    Draw the aggregate regime process Z conditional on H
    [draw.Z,draw.Sk] = HO_Z_CV(priors,draw);
    %    Draw the transition probabilities conditional on Z
    [draw.P] = HO_P_CV(priors,draw);

    %    Save the results
    ZED = zeros(priors.T,priors.K);
    for t = 1:priors.T;
        ZED(t,draw.Z(t,:)) = 1;
    end,
    results.ZED(:,:,i) = ZED;
    results.mu_0(:,i) = draw.mu_0;
    results.mu_1(:,i) = draw.mu_1;
    results.sigsq(:,i) = draw.sigsq;
    results.P(:,:,i) = draw.P;
    results.KAY(:,:,i) = draw.KAY;
    results.beta(:,:,i) = draw.beta;
end,
%[score] = HO_cross_val(R,KAPPA);
%   compile the results
[finalres,meanz] = HO_c_comp_CV(priors,results);
